@@ -1,8 +1,19 @@
 """Base class for all objects requiring a valid names for all engines."""
-from pydantic import BaseModel, Field
+from pydantic import BaseModel, Field, Extra
 
 
-class NamedBaseModel(BaseModel):
+class NoExtraBaseModel(BaseModel):
+    """Base class for all objects that are not extensible with additional keys.
+    
+    This effectively includes all objects except for the Properties classes
+    that are assigned to geometry objects.
+    """
+
+    class Config:
+        extra = Extra.forbid
+
+
+class NamedBaseModel(NoExtraBaseModel):
     """Base class for all objects requiring a valid names for all engines."""
 
     name: str = Field(
