@@ -1,9 +1,9 @@
 """Model schema and the 5 geometry objects that define it."""
-from pydantic import BaseModel, Field, validator, constr
+from pydantic import BaseModel, Field, validator, root_validator, constr, conlist
 from typing import List, Union
 from enum import Enum
 
-from ._base import NamedBaseModel
+from ._base import NamedBaseModel, NoExtraBaseModel
 from .bc import Outdoors, Surface, Ground, Adiabatic
 from .energy.properties import ShadeEnergyPropertiesAbridged, \
     DoorEnergyPropertiesAbridged, ApertureEnergyPropertiesAbridged, \
@@ -11,7 +11,7 @@ from .energy.properties import ShadeEnergyPropertiesAbridged, \
     ModelEnergyProperties
 
 
-class Plane(BaseModel):
+class Plane(NoExtraBaseModel):
 
     type: constr(regex='^Plane$') = 'Plane'
 
@@ -31,25 +31,26 @@ class Plane(BaseModel):
 
     x: List[float] = Field(
         default=None,
-        description="Plane x-axis as 3 (x, y, z) values",
+        description="Plane x-axis as 3 (x, y, z) values. If None, it is autocalculated.",
         min_items=3,
         max_items=3
     )
 
 
-class Face3D(BaseModel):
+class Face3D(NoExtraBaseModel):
     """A single planar face in 3D space."""
 
     type: constr(regex='^Face3D$') = 'Face3D'
 
-    boundary: List[List[float]] = Field(
+    boundary: List[conlist(float, min_items=3, max_items=3)] = Field(
         ...,
+        min_items=3,
         description='A list of points representing the outer boundary vertices of '
             'the face. The list should include at least 3 points and each point '
             'should be a list of 3 (x, y, z) values.'
     )
 
-    holes: List[List[List[float]]] = Field(
+    holes: List[conlist(conlist(float, min_items=3, max_items=3), min_items=3)] = Field(
         default=None,
         description='Optional list of lists with one list for each hole in the face.'
             'Each hole should be a list of at least 3 points and each point a list '
@@ -63,19 +64,6 @@ class Face3D(BaseModel):
             'If None, the plane will usually be derived from the boundary points.'
     )
 
-    @validator('boundary')
-    def check_num_items(cls, v):
-        for i in v:
-            assert len(i) == 3, 'Number of floats must be 3 for (x, y, z).'
-        return v
-
-    @validator('holes')
-    def check_num_items_holes(cls, v):
-        for pt_list in v:
-            for pt in pt_list:
-                assert len(pt) == 3, 'Number of floats must be 3 for (x, y, z).'
-        return v
-
 
 class ShadePropertiesAbridged(BaseModel):
 
@@ -101,6 +89,11 @@ class Shade(NamedBaseModel):
             '(Radiance, EnergyPlus).'
     )
 
+    user_data: dict = Field(
+        default={},
+        description='Optional dictionary of user data associated with the object.'
+    )
+
 
 class DoorPropertiesAbridged(BaseModel):
 
@@ -135,12 +128,28 @@ class Door(NamedBaseModel):
             'to an opaque door.'
     )
 
+    indoor_shades: List[Shade] = Field(
+        default=None,
+        description='Shades assigned to the interior side of this object.'
+    )
+
+    outdoor_shades: List[Shade] = Field(
+        default=None,
+        description='Shades assigned to the exterior side of this object '
+            '(eg. entryway awning).'
+    )
+
     properties: DoorPropertiesAbridged = Field(
         ...,
         description='Extension properties for particular simulation engines '
             '(Radiance, EnergyPlus).'
     )
 
+    user_data: dict = Field(
+        default={},
+        description='Optional dictionary of user data associated with the object.'
+    )
+
 
 class AperturePropertiesAbridged(BaseModel):
 
@@ -192,6 +201,11 @@ class Aperture(NamedBaseModel):
             '(Radiance, EnergyPlus).'
     )
 
+    user_data: dict = Field(
+        default={},
+        description='Optional dictionary of user data associated with the object.'
+    )
+
 
 class FacePropertiesAbridged(BaseModel):
 
@@ -207,7 +221,7 @@ class FaceType(str, Enum):
     wall = 'Wall'
     floor = 'Floor'
     roof_ceiling = 'RoofCeiling'
-    air_wall = 'AirWall'
+    air_boundary = 'AirBoundary'
 
 
 class Face(NamedBaseModel):
@@ -259,12 +273,28 @@ class Face(NamedBaseModel):
             '(Radiance, EnergyPlus).'
     )
 
+    user_data: dict = Field(
+        default={},
+        description='Optional dictionary of user data associated with the object.'
+    )
+
+    @root_validator
+    def chack_air_boundarys_are_interior(cls, values):
+        """Check that all air wall faces have a Surface boundary condition."""
+        face_type, bc = values.get('face_type'), values.get('boundary_condition')
+        if face_type == 'AirBoundary':
+            assert bc.type == 'Surface', \
+                'AirBoundarys must have "Surface" boundary conditions.'
+        return values
+
 
 class RoomPropertiesAbridged(BaseModel):
 
     type: constr(regex='^RoomPropertiesAbridged$') = 'RoomPropertiesAbridged'
 
-    energy: RoomEnergyPropertiesAbridged
+    energy: RoomEnergyPropertiesAbridged = Field(
+        default=None
+    )
 
 
 class Room(NamedBaseModel):
@@ -277,7 +307,7 @@ class Room(NamedBaseModel):
         description='Faces that together form the closed volume of a room.'
     )
 
-    indoor_shades:  List[Shade] = Field(
+    indoor_shades: List[Shade] = Field(
         default=None,
         description='Shades assigned to the interior side of this object '
             '(eg. partitions, tables).'
@@ -298,18 +328,33 @@ class Room(NamedBaseModel):
     multiplier: int = Field(
         1,
         ge=1,
-        description='Get or set an integer noting how many times this Room is repeated. '
+        description='An integer noting how many times this Room is repeated. '
             'Multipliers are used to speed up the calculation when similar Rooms are '
             'repeated more than once. Essentially, a given simulation with the '
             'Room is run once and then the result is mutliplied by the multiplier.'
     )
 
+    user_data: dict = Field(
+        default={},
+        description='Optional dictionary of user data associated with the object.'
+    )
+
+
+class Units(str, Enum):
+    meters = 'Meters'
+    millimeters = 'Millimeters'
+    feet = 'Feet'
+    inches = 'Inches'
+    centimeters = 'Centimeters'
+
 
 class ModelProperties(BaseModel):
 
     type: constr(regex='^ModelProperties$') = 'ModelProperties'
 
-    energy: ModelEnergyProperties
+    energy: ModelEnergyProperties = Field(
+        default=None
+    )
 
 
 class Model(NamedBaseModel):
@@ -354,12 +399,51 @@ class Model(NamedBaseModel):
         description='The clockwise north direction in degrees.'
     )
 
+    units: Units = Field(
+        default=Units.meters,
+        description='Text indicating the units in which the model geometry exists. '
+            'This is used to scale the geometry to the correct units for simulation '
+            'engines like EnergyPlus, which requires all geometry be in meters.'
+    )
+
+    tolerance: float = Field(
+        default=0,
+        description='The maximum difference between x, y, and z values at which '
+            'vertices are considered equivalent. This value should be in the Model '
+            'units and it is used in a variety of checks, including checks for '
+            'whether Room faces form a closed volume and subsequently correcting all '
+            'face normals point outward from the Room. A value of 0 will result '
+            'in no attempt to evaluate whether Room volumes are closed or check '
+            'face direction. So it is recommended that this always be a positive '
+            'number when such checks have not been performed on a Model. '
+            'Typical tolerances for builing geometry range from 0.1 to 0.0001 '
+            'depending on the units of the geometry.'
+    )
+
+    angle_tolerance: float = Field(
+        default=0,
+        description='The max angle difference in degrees that vertices are '
+            'allowed to differ from one another in order to consider them colinear. '
+            'This value is used in a variety of checks, including checks for '
+            'whether Room faces form a closed volume and subsequently correcting all '
+            'face normals point outward from the Room. A value of 0 will result '
+            'in no attempt to evaluate whether the Room volumes is closed or check '
+            'face direction. So it is recommended that this always be a positive '
+            'number when such checks have not been performed on a given Model. '
+            'Typical tolerances for builing geometry are often around 1 degree.'
+    )
+
     properties: ModelProperties = Field(
         ...,
         description='Extension properties for particular simulation engines '
             '(Radiance, EnergyPlus).'
     )
 
+    user_data: dict = Field(
+        default={},
+        description='Optional dictionary of user data associated with the object.'
+    )
+
 
 if __name__ == '__main__':
     print(Model.schema_json(indent=2))
